package com.cxytiandi.baidu_pan.repository;

import java.util.ArrayList;
import java.util.List;

import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.highlight.HighlightBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.elasticsearch.core.ElasticsearchTemplate;
import org.springframework.data.elasticsearch.core.ResultsExtractor;
import org.springframework.data.elasticsearch.core.query.NativeSearchQueryBuilder;
import org.springframework.stereotype.Repository;
import com.cxytiandi.baidu_pan.domain.es.ResourceBean;
import com.cxytiandi.baidu_pan.util.PageBean;
import com.cxytiandi.baidu_pan.util.ResourceTypeUtil;

@Repository
public class ResourceRepositoryImpl implements ResourceBeanRepository {
	@Autowired
	private ElasticsearchTemplate elasticsearchTemplate;
	
	@Override
	public PageBean<ResourceBean> listForPage(String keyword, int type, int page, int limit) {
		NativeSearchQueryBuilder query = new NativeSearchQueryBuilder();
		query.withIndices("cxytiandi_pan");
		query.withTypes("pan");
		query.withHighlightFields(
				new HighlightBuilder.Field("title").preTags("<font style='color:red;'>").postTags("</font>"));
		
		if (type > 0) {
			query.withQuery(
					QueryBuilders.boolQuery()
					.must(QueryBuilders.matchQuery("title", keyword))
					.must(QueryBuilders.termQuery("type", ResourceTypeUtil.getType(type))));
		} else {
			query.withQuery(
					QueryBuilders.boolQuery()
					.must(QueryBuilders.matchQuery("title", keyword))
			);
		}
		
		long total = elasticsearchTemplate.count(query.build());
		query.withPageable(new PageRequest(page - 1, limit));
		List<ResourceBean> list = elasticsearchTemplate.query(query.build(), new ResultsExtractor<List<ResourceBean>>() {
			@Override
			public List<ResourceBean> extract(SearchResponse response) {
				if (response.getHits().getHits().length <= 0) {
					return null;
				}
				List<ResourceBean> resultList = new ArrayList<>();
				for (SearchHit hit : response.getHits()) {
					ResourceBean bean = new ResourceBean();
					bean.setId(hit.getId());
					//有高亮就拿高量的值，没有就用原始的标题
					if (hit.getHighlightFields().get("title") != null) {
						bean.setTitle(hit.getHighlightFields().get("title").fragments()[0].toString());
					} else {
						bean.setTitle(hit.getSource().get("title").toString());
					}
					
					bean.setDate(hit.getSource().get("date").toString());
					bean.setDownloadCount(Long.parseLong(hit.getSource().get("downloadCount").toString()));
					bean.setHomeUrl(hit.getSource().get("homeUrl").toString());
					bean.setSize(hit.getSource().get("size").toString());
					bean.setType(hit.getSource().get("type").toString());
					bean.setUrl(hit.getSource().get("url").toString());
					bean.setUsername(hit.getSource().get("username").toString());
					bean.setVisitCount(Long.parseLong(hit.getSource().get("visitCount").toString()));
					resultList.add(bean);
				}
				return resultList;
			}
		});
		return new PageBean<>(PageBean.page2Start(page, limit), limit, list, total);
	}

}
